from django.apps import AppConfig


class EUTAppConfig(AppConfig):

    name = 'djangocms_misc.editmode_fallback_placeholder'

    def ready(self):
        from cms.utils import plugins
        from .plugins import assign_plugins
        from .plugins import move_plugin
        from cms.admin.placeholderadmin import PlaceholderAdminMixin

        plugins.assign_plugins = assign_plugins
        PlaceholderAdminMixin.original_move = PlaceholderAdminMixin.move_plugin
        PlaceholderAdminMixin.move_plugin = move_plugin



"""Settings that need to be set in order to run the tests."""
import os
import logging


DEBUG = True

logging.getLogger("factory").setLevel(logging.WARN)

SITE_ID = 1

# from selenium.webdriver.firefox import webdriver
# from selenium.webdriver.phantomjs import webdriver
# SELENIUM_WEBDRIVER = webdriver

APP_ROOT = os.path.abspath(
    os.path.join(os.path.dirname(__file__), ".."))

DATABASES = {
    'default': {
        'ENGINE': 'django.db.backends.sqlite3',
        'NAME': 'db.sqlite',
    }
}

LANGUAGE_CODE = 'en'
LANGUAGES = (
    ('en', 'English', ),
    ('de', 'Deutsch', ),
    ('fr', 'Frentsch', ),
)

# enable this if you think cache might screw something up.
# CACHES = {
#     'default': {
#         'BACKEND': 'django.core.cache.backends.dummy.DummyCache',
#     }
# }

ROOT_URLCONF = 'djangocms_misc.tests.urls'

MEDIA_ROOT = os.path.join(APP_ROOT, 'tests/test_app_media')
MEDIA_URL = "/media/"
STATIC_URL = '/static/'
STATIC_ROOT = os.path.join(APP_ROOT, '../test_app_static')
STATICFILES_DIRS = (
    os.path.join(APP_ROOT, 'static'),
)

# own settings!
DJANGOCMS_MISC_UNTRANSLATED_PLACEHOLDERS = 'en'
DJANGOCMS_MISC_GET_FROM_PAGE_CONTENT = {
    'video_id': {
        'placeholders': ('Inhalt',),
        'plugins': {
            'YoutubeVideoPlugin': ['video_id'],
        }
    },
    'image': {
        'placeholders': ('Inhalt',),
        'plugins': {
            'ImagePlugin': ('image', 'preview_image'),
            'HeaderPlugin': ('image',),
        }
    },
    'text': {
        'placeholders': ('translated_placeholder', 'untranslated_placeholder'),
        'plugins': {
            'TestPlugin': ('field1', ),
        }
    },
}

CMS_PLACEHOLDER_CONF = {
    'translated_placeholder': {
        'language_fallback': False,
    },
    'untranslated_placeholder': {
        'language_fallback': False,
        # 'untranslated': True,  # legacy version
        'untranslated': 'en',
    },
    'editmode_fallback_placeholder': {
        'language_fallback': True,
        'editmode_language_fallback': True,
    },
}

TEMPLATES = [
    {
        'BACKEND': 'django.template.backends.django.DjangoTemplates',
        # 'DIRS': [],
        'OPTIONS': {
            'context_processors': [
                'django.template.context_processors.debug',
                'django.contrib.auth.context_processors.auth',
                'django.contrib.messages.context_processors.messages',
                'django.template.context_processors.i18n',
                'django.template.context_processors.request',
                'django.template.context_processors.media',
                'django.template.context_processors.static',
                'sekizai.context_processors.sekizai',
                'cms.context_processors.cms_settings',

                'djangocms_misc.basic.context_processors.get_env',
            ],
            'loaders': [
                'django.template.loaders.filesystem.Loader',
                'django.template.loaders.app_directories.Loader',
                # 'django.template.loaders.eggs.Loader',
            ],
        }
    },
]

CMS_TEMPLATES = (
    ('base.html', 'Default'),
)

COVERAGE_REPORT_HTML_OUTPUT_DIR = os.path.join(
    os.path.join(APP_ROOT, 'tests/coverage'))
COVERAGE_MODULE_EXCLUDES = [
    'tests$', 'settings$', 'urls$', 'locale$',
    'migrations', 'fixtures', 'admin$', 'django_extensions',
]

EXTERNAL_APPS = [
    # 'django.contrib.admindocs',
    'django.contrib.auth',
    'django.contrib.contenttypes',
    'django.contrib.messages',
    'django.contrib.sessions',
    'django.contrib.staticfiles',
    'django.contrib.sitemaps',
    'django.contrib.sites',
    # cms
    'sekizai',
    'treebeard',
    'cms',
    'djangocms_link',
    'djangocms_text_ckeditor',
    'djangocms_admin_style',
    'menus',
    'filer',
    'mptt',
    'hvad',
    'modeltranslation',
    # 'ckeditor',

    'easy_thumbnails',
    'django.contrib.admin',
]

INTERNAL_APPS = [
    'djangocms_misc.basic',
    'djangocms_misc.admin_style',
    'djangocms_misc.alternate_toolbar',
    'djangocms_misc.global_untranslated_placeholder',
    # 'djangocms_misc.autopublisher',
    # 'djangocms_misc.untranslated_placeholder',
    # 'djangocms_misc.editmode_fallback_placeholder',

    'djangocms_misc.tests.test_app',
    # 'djangocms_misc.apphook_templates',
]

MIDDLEWARE = [
    'django.contrib.sessions.middleware.SessionMiddleware',
    'django.contrib.auth.middleware.AuthenticationMiddleware',
    'django.contrib.messages.middleware.MessageMiddleware',
    'django.middleware.csrf.CsrfViewMiddleware',
    'django.middleware.common.CommonMiddleware',
    'django.middleware.locale.LocaleMiddleware',
    'cms.middleware.user.CurrentUserMiddleware',
    'cms.middleware.page.CurrentPageMiddleware',
    'cms.middleware.toolbar.ToolbarMiddleware',
    'cms.middleware.language.LanguageCookieMiddleware',
    # Uncomment the next line for simple clickjacking protection:
    'djangocms_misc.basic.middleware.RedirectFirstSubpageMiddleware',
    'django.middleware.clickjacking.XFrameOptionsMiddleware',
    'django.middleware.locale.LocaleMiddleware',
]

INSTALLED_APPS = INTERNAL_APPS + EXTERNAL_APPS
COVERAGE_MODULE_EXCLUDES += EXTERNAL_APPS

SECRET_KEY = 'foobarXXXxxsvXY'



from .bot404 import Bot404Middleware
from .password_protected import PasswordProtectedMiddleware
from .redirect_subpage import RedirectFirstSubpageMiddleware


__all__ = [
    'Bot404Middleware',
    'PasswordProtectedMiddleware',
    'RedirectFirstSubpageMiddleware',
]



from django.shortcuts import redirect

# compat
import django
if django.VERSION[:2] < (1, 10):
    from django.core.urlresolvers import reverse
else:
    from django.urls import reverse


class PasswordProtectedMiddleware(object):

    def __init__(self, get_response=None):
        if get_response:
            self.get_response = get_response
        # One-time configuration and initialization.

    def __call__(self, request):
        # Code to be executed for each request before
        # the view (and later middleware) are called.
        a_possible_redirect = self.process_request(request)
        if a_possible_redirect:
            return a_possible_redirect
        response = self.get_response(request)
        # Code to be executed for each request/response after
        # the view is called.
        return response

    def process_request(self, request):
        # raise a 404 if bot and enabled.
        login_url = reverse('admin:login')
        logout_url = reverse('admin:logout')
        if request.path in (login_url, logout_url,):
            return
        if not request.user.is_authenticated:
            redirect_url = '{}?next={}'.format(login_url, request.path)
            return redirect(redirect_url)



from django.conf import settings
from django.http import Http404


# try to be old and new style middleware compatible
class Bot404Middleware(object):
    def __init__(self, get_response=None):
        if get_response:
            self.get_response = get_response
        # One-time configuration and initialization.

    def __call__(self, request):
        # Code to be executed for each request before
        # the view (and later middleware) are called.
        self.process_request(request)
        response = self.get_response(request)
        # Code to be executed for each request/response after
        # the view is called.
        return response

    def process_request(self, request):
        # raise a 404 if bot and enabled.
        enabled = getattr(settings, 'DJANGOCMS_MISC_BOT404', None)
        user_agent = getattr(request, 'user_agent', None)
        if enabled and user_agent and user_agent.is_bot:
            raise Http404



from django.contrib import admin

from .models import TestModel, TestInlineModel


class TestInlineModelInline(admin.StackedInline):
    model = TestInlineModel
    extra = 2


class TestInlineModelInline2(admin.TabularInline):
    model = TestInlineModel
    extra = 2


class TestModelAdmin(admin.ModelAdmin):
    inlines = [TestInlineModelInline, TestInlineModelInline2, ]
    fieldsets = [
        ['', {'fields': ['field0', ]}],
        ['First Set', {'fields': ['field1', 'field2']}],
    ]


admin.site.register(TestModel, TestModelAdmin)



# coding: utf-8
from django.conf import settings
from cms.models import Placeholder  # noqa - needed, circular import otherwise
from cms.plugin_rendering import ContentRenderer  # , RenderedPlaceholder

# load conf at startup
from .conf import UntranslatedPlaceholderConf  # noqa
# import signals at startup
from .signals import *  # noqa (will forget to update otherwise!)


def content_renderer__init__(self, request):
    self.__original_init__(request)
    global_untranslated = getattr(settings, 'DJANGOCMS_MISC_UNTRANSLATED_PLACEHOLDERS', None)
    if global_untranslated:
        if global_untranslated in settings.LANGUAGES:
            self.request_language = global_untranslated
        else:
            self.request_language = settings.LANGUAGE_CODE


# hope this is the way to do it!
ContentRenderer.__original_init__ = ContentRenderer.__init__
ContentRenderer.__init__ = content_renderer__init__



from django.conf import settings


def get_untranslated_default_language():
    value = getattr(settings, 'DJANGOCMS_MISC_UNTRANSLATED_PLACEHOLDERS', None)
    if value and value is not True:
        for lang_tuple in settings.LANGUAGES:
            if lang_tuple[0] == value:
                return value
    return settings.LANGUAGE_CODE



# coding: utf-8
from __future__ import unicode_literals

from django.conf import settings
from django.contrib.auth import get_user_model, get_permission_codename
from django.contrib import admin
from django.utils.translation import ugettext_lazy as _
from cms.utils.urlutils import admin_reverse
from cms.toolbar_pool import toolbar_pool
from cms.cms_toolbars import BasicToolbar, ADMIN_MENU_IDENTIFIER, ADMINISTRATION_BREAK

from djangocms_misc.utils.edit_mode import is_edit_or_build_mode

USER_MENU_IDENTIFIER = 'user-menu'
CLIPBOARD_MENU_IDENTIFIER = 'clipboard-menu'


toolbar_pool.unregister(BasicToolbar)


@toolbar_pool.register
class AlternateBasicToolbar(BasicToolbar):

    def populate(self):
        # still dont know why this if!?
        if not self.page:
            self.init_from_request()
            self.clipboard = self.request.toolbar.user_settings.clipboard
            self.add_user_menu()
            self.add_admin_menu()
            self.add_clipboard_menu()
            self.add_language_menu()

    def add_user_menu(self):
        # menu
        self.user_menu = self.toolbar.get_or_create_menu(
            USER_MENU_IDENTIFIER,
            _('User'),
            position=0,
        )
        # buttons / items
        self.user_menu.add_sideframe_item(
            _("Change Password"),
            url=admin_reverse('password_change'),
        )
        self.user_menu.add_sideframe_item(
            _('User settings'),
            url=admin_reverse('cms_usersettings_change'),
        )
        self.add_logout_button(self.user_menu)

    def add_admin_menu(self):
        # menu
        self.admin_menu = self.toolbar.get_or_create_menu(
            ADMIN_MENU_IDENTIFIER,
            _('Administration'),
            position=1,
        )
        # buttons / items (pages are added automagically, in PageToolbar!)
        if 'filer' in settings.INSTALLED_APPS:
            self.admin_menu.add_sideframe_item(
                _('Files'),
                url=admin_reverse('filer_folder_changelist')
            )
        # in between
        self.add_more_admin_menu_items()
        # end
        self.admin_menu.add_break(ADMINISTRATION_BREAK, position=199)
        self.add_user_group_button(self.admin_menu, position=200)
        self.admin_menu.add_sideframe_item(
            _('Administration'),
            url=admin_reverse('index'),
            position=200
        )

    # override
    def add_more_admin_menu_items(self):
        pass

    def add_user_group_button(self, parent, position=200):
        User = get_user_model()
        if User in admin.site._registry:
            opts = User._meta
            if self.request.user.has_perm(
                    '%s.%s' % (opts.app_label, get_permission_codename('change', opts))):
                user_group_changelist_url = admin_reverse(
                    'app_list', args=(opts.app_label, ))
                parent.add_sideframe_item(
                    _('Users & Groups'),
                    url=user_group_changelist_url,
                    position=position
                )

    def add_clipboard_menu(self):
        # menu
        self.clipboard_menu = self.toolbar.get_or_create_menu(
            CLIPBOARD_MENU_IDENTIFIER,
            _('Clipboard'),
            position=-1,
        )
        # buttons / items
        # if self.toolbar.edit_mode or getattr(self.toolbar, 'build_mode', None):
        # new, testing for preventing a recursion error!
        if (is_edit_or_build_mode(self)
            and getattr(self.request, 'current_page', None)
        ):
            # True if the clipboard exists and there's plugins in it.
            if getattr(self, 'get_clipboard_plugins', None):
                # cms up to 4.4.6
                if getattr(self, 'clipboard_plugin_prevent_recursion', None):
                    # recursive loop, when empty clipboard?!
                    clipboard_is_bound = False
                else:
                    self.clipboard_plugin_prevent_recursion = True
                    clipboard_is_bound = self.get_clipboard_plugins().exists()
            else:
                clipboard_is_bound = self.toolbar.clipboard_plugin
            self.clipboard_menu.add_link_item(
                _('Clipboard...'),
                url='#',
                extra_classes=['cms-clipboard-trigger'],
                disabled=not clipboard_is_bound,
            )
            self.clipboard_menu.add_link_item(
                _('Clear clipboard'),
                url='#',
                extra_classes=['cms-clipboard-empty'],
                disabled=not clipboard_is_bound,
            )



# -*- coding: utf-8 -*-
from cms.api import create_page, create_title, add_plugin
from django.test import Client
from django.test.testcases import TestCase

from djangocms_misc.tests.test_app.cms_plugins import TestPlugin


class UntranslatedPlaceholderTestCase(TestCase):

    def setUp(self):
        self.client = Client()
        # u = self._create_user("test", True, True)
        # self._login_context = self.login_user_context(u)
        # self._login_context.__enter__()

    def tearDown(self):
        pass
        # self._login_context.__exit__(None, None, None)

    def test_basic(self):
        """ Tests untranslated placeholder configuration """
        page = create_page('page_en', 'base.html', 'en')
        create_title("de", "page_de", page)
        placeholder_en = page.placeholders.get(slot='untranslated_placeholder')
        add_plugin(placeholder_en, TestPlugin, 'en', field1='en field1')
        page.publish('en')
        page.publish('de')

        # English page should have the text plugin
        content_en = self.client.get(page.get_absolute_url())
        self.assertRegexpMatches(str(content_en.content), "en field1")
        # Deutsch page have text due to untranslated
        content_de = self.client.get(page.get_absolute_url('de'))
        self.assertRegexpMatches(str(content_de.content), "en field1")

    def test_publish_non_default_language(self):
        """
        test if a publish in a no default language also works
        """
        page = create_page('page_en', 'base.html', 'en')
        create_title("de", "page_de", page)
        placeholder_en = page.placeholders.get(slot='untranslated_placeholder')
        plugin = add_plugin(placeholder_en, TestPlugin, 'en', field1='starting different')
        page.publish('en')
        page.publish('de')
        plugin.field1 = 'en field1'
        plugin.save()
        page.publish('de')

        # English page should have the text plugin
        content_en = self.client.get(page.get_absolute_url())
        self.assertRegexpMatches(str(content_en.content), "en field1")
        # Deutsch page have text due to untranslated
        content_de = self.client.get(page.get_absolute_url('de'))
        self.assertRegexpMatches(str(content_de.content), "en field1")





def is_edit_mode(toolbar):
    if (getattr(toolbar, 'edit_mode', None)  # cms pre 3.6
        or getattr(toolbar, 'edit_mode_active', None)  # cms 3.6+
    ):
        return True
    return False


def is_edit_or_build_mode(toolbar):
    if (getattr(toolbar, 'edit_mode', None)  # cms pre 3.6
        or getattr(toolbar, 'edit_mode_active', None)  # noqa cms 3.6+
        or getattr(toolbar, 'build_mode', None)  # noqa
        or getattr(toolbar, 'build_mode_active', None)  # noqa
    ):
        return True
    return False



# -*- coding: utf-8 -*-
# Generated by Django 1.11.28 on 2020-02-11 16:15
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('test_app', '0004_testpluginmodel_field1_fr'),
    ]

    operations = [
        migrations.AddField(
            model_name='testpluginmodel',
            name='field_date',
            field=models.DateField(default=None, null=True),
        ),
        migrations.AddField(
            model_name='testpluginmodel',
            name='field_datetime',
            field=models.DateTimeField(default=None, null=True),
        ),
        migrations.AddField(
            model_name='testpluginmodel',
            name='field_time',
            field=models.TimeField(default=None, null=True),
        ),
    ]



from __future__ import unicode_literals

# from cms.admin.static_placeholder import StaticPlaceholderAdmin
# from django.conf import settings
# from django.urls import resolve
from cms.models import CMSPlugin, StaticPlaceholder
from cms.signals import post_placeholder_operation  # post_obj_operation
from django.db.models.signals import post_save
from django.dispatch import receiver

# ugly
from cms import __version__ as cms_version
CMS_VERSION_36 = cms_version.startswith('3.6.')


def check_publish(title_obj, force_non_dirty=False):
    page = title_obj.page
    if title_obj.published and page.publisher_is_draft:
        # print("published and draft!")
        # print(title_obj.is_dirty())
        # not dirty, after plugin add, in cms 3.6!
        if title_obj.is_dirty() or force_non_dirty or CMS_VERSION_36:
            # print("NEEEEEDs publishing")
            page.publish(title_obj.language)
            # from cms.api import publish_page
            # publish_page(page, user, title_obj.language)
            # print("done publishing")
    else:
        pass


@receiver(
    post_save,
    dispatch_uid="cms_autopublisher_publish_check_save_plugin_instance",
)
def cms_plugin_instance_post_save(sender, instance, **kwargs):
    # print("post save whatever")
    # print(sender)
    created = kwargs.get('created')
    # print(created)
    if created and issubclass(sender, CMSPlugin):
        page = getattr(instance.placeholder, 'page', None)
        if page:
            title = page.get_title_obj(instance.language)
            check_publish(title)


# @receiver(
#     post_delete,
#     dispatch_uid="cms_autopublisher_publish_check_post_delete_plugin_instance",
# )
# def cms_plugin_instance_post_delete(sender, instance, **kwargs):
#     if issubclass(sender, CMSPlugin):
#         page = instance.placeholder.page
#         if page:
#             title = page.get_title_obj(instance.language)
#             check_publish(title)


# ATTENTION ! ***** is called BEFORE page is marked dirty!!!
# works well for move operations, but not perfect for copy paste! (order is not taken)
# source of signals:
@receiver(
    post_placeholder_operation,
    dispatch_uid="cms_autopublisher_post_placeholder_operation",
)
def check_post_placeholder_operation(sender, operation, request, language, token, origin, **kwargs):
    print("post placeholder operation!")
    # print(sender)
    # print(operation)
    # print(request)
    # print(language)
    # print(kwargs)
    # print(origin)
    plugin = None
    placeholder = None
    language = None
    if operation == 'move_plugin':
        plugin = kwargs.get('plugin', None)
    # solved with above post_save signal
    # if operation == 'add_plugin':
    #     plugin = kwargs.get('plugin', None)
    if operation == 'delete_plugin':
        plugin = kwargs.get('plugin', None)
    if operation == 'paste_plugin':
        plugin = kwargs.get('plugin', None)
    if operation == 'change_plugin':
        plugin = kwargs.get('new_plugin', None)
    if operation == 'cut_plugin':
        placeholder = kwargs.get('source_placeholder', None)
        the_plugin = kwargs.get('plugin', None)
        language = the_plugin.language
    if operation == 'paste_placeholder':
        plugin = kwargs.get('plugins', [None, ])[0]
    if operation == 'clear_placeholder':
        plugin = kwargs.get('plugins', [None, ])[0]

    if plugin:
        placeholder = plugin.placeholder
        language = plugin.language
    if placeholder:
        page = getattr(placeholder, 'page', None)
        if page:
            title = page.get_title_obj(language)
            check_publish(title, force_non_dirty=True)
        else:
            if placeholder._get_attached_model() == StaticPlaceholder:
                # print("static placeholder!")
                attached_objs = placeholder._get_attached_objects()
                if len(attached_objs) == 1:
                    attached_objs[0].publish(None, language, force=True)
        return


# # TODO: can Title instances still be deleted? or just be unpublished, what would be covered here?
@receiver(
    post_save,
    sender='cms.Title',
    dispatch_uid="cms_autopublisher_publish_check_save_title",
)
def check_title_post_save(sender, instance, **kwargs):
    # print("cms Title check")
    check_publish(instance)


# @receiver(
#     post_save,
#     sender='cms.Page',
#     dispatch_uid="cms_autopublisher_publish_check_save_title",
# )
# def cms_page_post_save(sender, instance, **kwargs):
#     print "cms Page check. marking as dirty is done on TItle object!"
#     for title in instance.title_set.all():
#         check_publish(title)



# -*- coding: utf-8 -*-
# Generated by Django 1.11.12 on 2018-05-23 08:10
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('test_app', '0003_auto_20171014_0359'),
    ]

    operations = [
        migrations.AddField(
            model_name='testpluginmodel',
            name='field1_fr',
            field=models.CharField(default='', max_length=64, null=True),
        ),
    ]



from __future__ import unicode_literals
from django.conf import settings


def get_env(request):
    """
    expose is_live/stage/dev and SITE_ID to context
    """
    context = {
        'SITE_ID': settings.SITE_ID,
    }
    env = getattr(settings, 'ENV', None)
    if env:
        context.update({
            'ENV': settings.ENV,
            'is_' + str(env): True,
        })
    return context



from django.contrib.auth.models import User


def create_superuser():
    superuser = User.objects.create_superuser('admin',
                                              'admin@free.fr',
                                              'secret')
    return superuser



# -*- coding: utf-8 -*-
from collections import defaultdict
from itertools import groupby
from operator import attrgetter

from cms.utils import get_language_from_request
from cms.utils.i18n import get_fallback_languages
from cms.utils.moderator import get_cmsplugin_queryset
from cms.utils.placeholder import get_placeholder_conf
from cms.utils.plugins import create_default_plugins, build_plugin_tree
from cms.utils.plugins import downcast_plugins
from django.db import transaction
from django.utils.decorators import method_decorator
from django.views.decorators.clickjacking import xframe_options_sameorigin
from django.views.decorators.http import require_POST

from djangocms_misc.utils.edit_mode import is_edit_mode


@method_decorator(require_POST)
@xframe_options_sameorigin
@transaction.atomic
def move_plugin(self, request):
    from cms.admin.placeholderadmin import get_int
    try:
        plugin_id = get_int(request.POST.get('plugin_id'))
    except TypeError:
        raise RuntimeError("'plugin_id' is a required parameter.")
    plugin = self._get_plugin_from_id(plugin_id)
    print(plugin.placeholder.slot)
    # print self
    # request.POST['plugin_language'] = plugin.language
    self.original_move_plugin(request)


def assign_plugins(request, placeholders, template, lang=None, is_fallback=False):
    """
    Fetch all plugins for the given ``placeholders`` and
    cast them down to the concrete instances in one query
    per type.
    """
    if not placeholders:
        return
    placeholders = tuple(placeholders)
    lang = lang or get_language_from_request(request)
    qs = get_cmsplugin_queryset(request)
    qs = qs.filter(placeholder__in=placeholders, language=lang)
    plugins = list(qs.order_by('placeholder', 'path'))
    fallbacks = defaultdict(list)
    # If no plugin is present in the current placeholder we loop in the fallback languages
    # and get the first available set of plugins
    # monkey patch: if condition changed, to allow fallbacks in edit mode
    # strange condition in original: and not hasattr(request, 'toolbar')
    if not is_fallback:
        disjoint_placeholders = (ph for ph in placeholders
                                 if all(ph.pk != p.placeholder_id for p in plugins))
        for placeholder in disjoint_placeholders:
            language_fallback = get_placeholder_conf(
                "language_fallback",
                placeholder.slot,
                template,
                True
            )
            editmode_language_fallback = get_placeholder_conf(
                "editmode_language_fallback",
                placeholder.slot,
                template,
                False
            )
            # monkey patch: check if we should display fallbacks in edit mode!
            if language_fallback and (not is_edit_mode(request.toolbar) or editmode_language_fallback):
                for fallback_language in get_fallback_languages(lang):
                    assign_plugins(
                        request,
                        (placeholder, ),
                        template,
                        fallback_language,
                        is_fallback=True,
                    )
                    fallback_plugins = placeholder._plugins_cache
                    if fallback_plugins:
                        fallbacks[placeholder.pk] += fallback_plugins
                        break
    # These placeholders have no fallback
    non_fallback_phs = [ph for ph in placeholders if ph.pk not in fallbacks]
    # If no plugin is present in non fallback placeholders, create default plugins if enabled)
    if not plugins:
        plugins = create_default_plugins(request, non_fallback_phs, template, lang)
    plugins = downcast_plugins(plugins, non_fallback_phs, request=request)
    # split the plugins up by placeholder
    # Plugins should still be sorted by placeholder
    plugin_groups = dict((key, list(plugins))
                         for key, plugins in groupby(plugins, attrgetter('placeholder_id')))
    all_plugins_groups = plugin_groups.copy()
    for group in plugin_groups:
        plugin_groups[group] = build_plugin_tree(plugin_groups[group])
    groups = fallbacks.copy()
    groups.update(plugin_groups)
    for placeholder in placeholders:
        # This is all the plugins.
        setattr(placeholder, '_all_plugins_cache', all_plugins_groups.get(placeholder.pk, []))
        # This one is only the root plugins.
        setattr(placeholder, '_plugins_cache', groups.get(placeholder.pk, []))



# coding: utf-8
from __future__ import unicode_literals

from cms.models import Page, Placeholder
from django.conf import settings
from django import template

from djangocms_misc.utils.edit_mode import is_edit_mode

register = template.Library()


@register.filter()
def djangocms_misc_placeholder_empty(page_placeholder, slot=None):
    """
    for page/slot, pass a page object, and a slot name:
    {% if request.current_page|djangocms_misc_placeholder_empty:"content" %}

    for a outside page placeholder, just the placeholder object:
    {% if object.placeholderfield|djangocms_misc_placeholder_empty %}

    also, with:
    {% with ph_empty=object.placeholderfield|djangocms_misc_placeholder_empty %}
    """
    placeholder = None
    if isinstance(page_placeholder, Placeholder):
        placeholder = page_placeholder
    elif isinstance(page_placeholder, Page):
        page = page_placeholder
        try:
            placeholder = page.placeholders.get(slot=slot)
        except Placeholder.DoesNotExist:
            pass
    if placeholder:
        # // return not placeholder.cmsplugin_set.filter(language=get_language()).exists()
        return not placeholder.cmsplugin_set.exists()
    return False


@register.inclusion_tag('djangocms_misc/tags/page_link.html', takes_context=True)
def djangocms_misc_page_link(context, lookup, css_class='', link_text='', link_text_attr=''):
    """
    link_text_attr is not working (yet)
    """
    if not link_text_attr:
        link_text_attr = 'title'
    context.update({
        'lookup': lookup,
        'css_class': css_class,
        'link_text': link_text,
        'link_text_attr': link_text_attr,
    })
    return context


@register.simple_tag(takes_context=True)
def djangocms_misc_get_from_page_content(context, config_name, page_lookup=None):
    config = settings.DJANGOCMS_MISC_GET_FROM_PAGE_CONTENT.get(config_name, None)
    request = context['request']
    page = None
    if isinstance(page_lookup, Page):
        page = page_lookup
    else:
        try:
            page_id = int(page_lookup)
            page = Page.objects.get(pk=page_id)
        except (TypeError, ValueError, Page.DoesNotExist):
            pass
        try:
            page_reverse_id = str(page_lookup)
            qs = Page.objects.all()
            if is_edit_mode(request.toolbar):
                qs = qs.drafts()
            else:
                qs = qs.public()
            page = qs.get(reverse_id=page_reverse_id)
        except (ValueError, Page.DoesNotExist):
            pass
        if not page:
            page = getattr(request, 'current_page', None)
    if page and config:
        content = get_from_page_content(request, config, page)
        return content
    return ''


def get_from_page_content(request, config, page):
    placeholders = page.get_placeholders()
    to_scan_placeholders = config.get('placeholders')
    to_scan_plugins = config.get('plugins')
    for slot_name in to_scan_placeholders:
        placeholder = placeholders.filter(slot=slot_name)
        if placeholder.count():
            language = request.LANGUAGE_CODE
            if 'djangocms_misc.global_untranslated_placeholder' in settings.INSTALLED_APPS and\
                    settings.DJANGOCMS_MISC_UNTRANSLATED_PLACEHOLDERS:
                from djangocms_misc.global_untranslated_placeholder.utils import \
                    get_untranslated_default_language
                language = get_untranslated_default_language()
            plugins = placeholder[0].get_plugins(language).order_by('position')
            for plugin in plugins:
                if plugin.plugin_type in to_scan_plugins:
                    instance, plugin_cls = plugin.get_plugin_instance()
                    to_scan_fields = to_scan_plugins[plugin.plugin_type]
                    if len(to_scan_fields):
                        for field in to_scan_fields:
                            if instance and getattr(instance, field, None):
                                content = getattr(instance, field)
                                return content
                    else:
                        return instance
    return ''



from modeltranslation.translator import translator, TranslationOptions
from .models import TestPluginModel


class TestPluginModelTranslationOptions(TranslationOptions):
    fields = ('field1', )


translator.register(TestPluginModel, TestPluginModelTranslationOptions)



from appconf import AppConf


# TODO: dont need a that complex one for default
class UntranslatedPlaceholderConf(AppConf):
    GET_FROM_PAGE_CONTENT = {
        'video_id': {
            'placeholders': ('Inhalt',),
            'plugins': {
                'YoutubeVideoPlugin': ['video_id'],
            }
        },
        'image': {
            'placeholders': ('Inhalt', ),
            'plugins': {
                'ImagePlugin': ('image', 'preview_image'),
                'HeaderPlugin': ('image', ),
            }
        },
        'text': {
            'placeholders': ('Inhalt',),
            'plugins': {
                'ImagePlugin': ('image', 'preview_image'),
                'HeaderPlugin': ('image',),
            }
        },
    }

    class Meta:
        prefix = 'djangocms_misc'



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from cms.api import create_page, create_title, add_plugin
from django.contrib.auth.models import User
from django.test import TestCase, Client, modify_settings

from djangocms_misc.basic.templatetags.djangocms_misc_tags import djangocms_misc_placeholder_empty
from djangocms_misc.tests.test_app.cms_plugins import TestPlugin


class BasicAppTests(TestCase):
    def setUp(self):
        self.client = Client()
        self.user = User.objects.create_superuser(
            username='fred',
            password='test',
            email='test@test.fred',
        )

    def tearDown(self):
        pass

    def test_page_link_tag(self):
        page_test = create_page('test', 'base.html', 'en')
        page_test.publish('en')
        page_home = create_page('home', 'base.html', 'en')
        page_home.reverse_id = 'home'
        page_home.save()
        page_home.publish('en')
        response = self.client.get(page_test.get_absolute_url('en'))
        self.assertContains(response, '/en/home/')
        self.assertContains(response, '/home/">link text HOME')
        self.assertContains(response, 'class="button" href="/en/home/">')

    def test_placeholder_empty_tag(self):
        page = create_page('page_en', 'base.html', 'en')
        page.reverse_id = 'test'
        create_title("de", "page_de", page)
        placeholder_en = page.placeholders.get(slot='untranslated_placeholder')
        self.assertEqual(djangocms_misc_placeholder_empty(placeholder_en), True)
        self.assertEqual(djangocms_misc_placeholder_empty(page, 'untranslated_placeholder'), True)
        plugin = add_plugin(placeholder_en, TestPlugin, 'en', )
        # plugin.field1 = 'en field1'
        plugin.field1_en = 'en field1'
        plugin.save()
        self.assertEqual(djangocms_misc_placeholder_empty(placeholder_en), False)
        self.assertEqual(djangocms_misc_placeholder_empty(page, 'untranslated_placeholder'), False)

    def test_get_from_page_content_tag(self):
        """
        Tests if content is fetched
        """
        page = create_page('page_en', 'base.html', 'en')
        page.reverse_id = 'test'
        create_title("de", "page_de", page)
        placeholder_en = page.placeholders.get(slot='untranslated_placeholder')
        plugin = add_plugin(placeholder_en, TestPlugin, 'en', )
        # plugin.field1 = 'en field1'
        plugin.field1_en = 'en field1'
        plugin.save()
        page.publish('en')
        page.publish('de')
        # untranslated placeholder is enabled, so the content shoudl appear de/en 2x each
        response = self.client.get(page.get_absolute_url('en'))
        self.assertContains(response, 'en field1', 5)
        response = self.client.get(page.get_absolute_url('de'))
        self.assertContains(response, 'en field1', 5)
        # also, draft mode!
        self.client.login(username='fred', password='test')
        response = self.client.get(page.get_absolute_url('en') + '?edit')
        # one more, in the structure mode/plugin representation
        self.assertContains(response, ': en field1', 5)

    def test_language_tabs_admin_mixin(self):
        # TODO: language tabs tests
        pass

    def test_redirect_first_subpage_middleware(self):
        page_home = create_page('home', 'base.html', 'en')
        page_home.publish('en')
        page_parent = create_page('parent', 'base.html', 'en', redirect='/firstchild')
        page_parent.publish('en')
        page_child1 = create_page('child1', 'base.html', 'en', parent=page_parent)
        page_child1.publish('en')
        page_child2 = create_page('child2', 'base.html', 'en', parent=page_parent)
        page_child2.publish('en')
        # get the parent
        response = self.client.get(page_parent.get_absolute_url('en'))
        self.assertEqual(response.status_code, 301)
        self.assertEqual(response.url, '/en/parent/child1/')
        # check for some false positives
        response = self.client.get(page_child1.get_absolute_url('en'))
        self.assertEqual(response.status_code, 200)
        response = self.client.get(page_home.get_absolute_url('en'))
        self.assertEqual(response.status_code, 200)
        response = self.client.get(page_child2.get_absolute_url('en'))
        self.assertEqual(response.status_code, 200)
        # 404 still ok?
        response = self.client.get('/en/absolutely-not-exising/22/')
        self.assertEqual(response.status_code, 404)

    @modify_settings(MIDDLEWARE={
        'append': 'djangocms_misc.basic.middleware.PasswordProtectedMiddleware',
    })
    def test_password_protected_middleware(self):
        page_home = create_page('home', 'base.html', 'en')
        page_home.reverse_id = 'home'
        page_home.save()
        page_home.publish('en')
        response = self.client.get(page_home.get_absolute_url('en'))
        self.assertEqual(response.status_code, 302)
        pass

    def test_bot404_middleware(self):
        # TODO: tests
        pass




from cms.utils import plugins
from .plugins import assign_plugins
from .plugins import move_plugin
from cms.admin.placeholderadmin import PlaceholderAdminMixin

plugins.assign_plugins = assign_plugins
PlaceholderAdminMixin.original_move = PlaceholderAdminMixin.move_plugin
PlaceholderAdminMixin.move_plugin = move_plugin



# -*- coding: utf-8 -*-
from cms.test_utils.testcases import CMSTestCase
from django.test import TestCase, Client, modify_settings

from djangocms_misc.tests.test_app.cms_plugins import TestPlugin


@modify_settings(INSTALLED_APPS={
    'append': 'djangocms_misc.autopublisher',
})
class AutoPublisherTestCase(CMSTestCase, TestCase):
    def setUp(self):
        self.client = Client()
        self._admin_user = self.get_superuser()

    def tearDown(self):
        pass

    def test_save_page(self):
        from cms import api as cms_api
        page = cms_api.create_page('test', 'base.html', 'en', reverse_id='what')
        page.publish('en')
        page.reverse_id = 'lucky'
        title_obj = page.get_title_obj('en')
        title_obj.title = 'dummy'
        title_obj.save()
        self.assertEqual(page.publisher_is_draft, True)
        # public_obj = page.get_public_object()
        # TODO: this isnt working yet!? normaly, a page is saved with a title. works via frontend, though!?
        # so when using the cms via frontend it works.
        # self.assertEqual(public_obj.reverse_id, 'lucky')

    def test_save_title(self):
        from cms import api as cms_api
        page = cms_api.create_page('test', 'base.html', 'en')
        page.publish('en')
        title = page.get_title_obj('en')
        title.title = 'another'
        title.save()
        self.assertEqual(page.publisher_is_draft, True)
        self.assertEqual(page.get_public_object().get_title_obj('en').title, 'another')
        pass

    def test_add_save_plugin(self):
        from cms import api as cms_api
        page = cms_api.create_page('test', 'base.html', 'en')
        page.publish('en')
        placeholder = page.placeholders.get(slot='translated_placeholder')
        plugin = cms_api.add_plugin(placeholder, TestPlugin, 'en', field1='teststring')
        # public_obj = page.get_public_object()
        response = self.client.get(page.get_absolute_url())
        self.assertRegexpMatches(str(response.content), "teststring")
        plugin.field1 = "another test"
        plugin.save()
        # TODO: inconsistency, publish should not be needed, but appearently, dirty state is not applied when saving
        # a plugin with save() only.
        page.publish('en')
        response = self.client.get(page.get_absolute_url())
        self.assertRegexpMatches(str(response.content), "another test")

    def test_move_plugin(self):
        from cms import api as cms_api
        page = cms_api.create_page('test', 'base.html', 'en')
        page.publish('en')
        placeholder = page.placeholders.get(slot='translated_placeholder')
        plugin1 = cms_api.add_plugin(placeholder, TestPlugin, 'en', field1='teststring')
        plugin2 = cms_api.add_plugin(placeholder, TestPlugin, 'en', field1='teststring2')
        uri = self.get_move_plugin_uri(
            plugin1,
            page,
            'en',
        )
        data = {
            'plugin_id': plugin1.id,
            'target_language': 'en',
            'placeholder_id': placeholder.id,
            'plugin_order': [plugin2.id, plugin1.id, ],
        }
        with self.login_user_context(self._admin_user):
            response = self.client.post(uri, data)
            self.assertEqual(response.status_code, 200)
        response = self.client.get(page.get_absolute_url())
        # helps debugging, when plugin output might have change slightly!
        # print(response.content)
        self.assertContains(response, '''<div>
    testplugin content: teststring2
</div><div>
    testplugin content: teststring
</div>
''')

    def test_move_page(self):
        pass

    def test_save_static_placeholder_plugin(self):
        pass

    def test_move_static_placeholder_plugin(self):
        pass



# -*- coding: utf-8 -*-
from collections import defaultdict
from itertools import groupby
from operator import attrgetter

from cms.utils import get_language_from_request
from cms.utils.i18n import get_fallback_languages
from cms.utils.moderator import get_cmsplugin_queryset
from cms.utils.placeholder import get_placeholder_conf
from cms.utils.plugins import create_default_plugins, build_plugin_tree
from cms.utils.plugins import downcast_plugins

from djangocms_misc.utils.edit_mode import is_edit_mode


def assign_plugins(request, placeholders, template, lang=None, is_fallback=False):
    """
    Fetch all plugins for the given ``placeholders`` and
    cast them down to the concrete instances in one query
    per type.
    """
    if not placeholders:
        return
    placeholders = tuple(placeholders)
    lang = lang or get_language_from_request(request)
    translated_placeholders = []
    untranslated_placeholders = []
    for placeholder in placeholders:
        if get_placeholder_conf("untranslated", placeholder.slot, template, False):
            untranslated_placeholders.append(placeholder)
        else:
            translated_placeholders.append(placeholder)
    _assign_plugins(request, translated_placeholders, template, lang, is_fallback)
    _assign_plugins(request, untranslated_placeholders, template, lang, is_fallback, True)


def _assign_plugins(request, placeholders, template, lang=None, is_fallback=False,
                    untranslated=False):
    qs = get_cmsplugin_queryset(request)
    if untranslated:
        qs = qs.filter(placeholder__in=placeholders)
    else:
        qs = qs.filter(placeholder__in=placeholders, language=lang)
    plugins = list(qs.order_by('placeholder', 'path'))
    fallbacks = defaultdict(list)
    # If no plugin is present in the current placeholder we loop in the fallback languages
    # and get the first available set of plugins
    if not (is_fallback or untranslated) \
            and not (hasattr(request, 'toolbar') and is_edit_mode(request.toolbar)):
        disjoint_placeholders = (ph for ph in placeholders
                                 if all(ph.pk != p.placeholder_id for p in plugins))
        for placeholder in disjoint_placeholders:
            if get_placeholder_conf("language_fallback", placeholder.slot, template, True):
                for fallback_language in get_fallback_languages(lang):
                    _assign_plugins(
                        request,
                        (placeholder,),
                        template,
                        fallback_language,
                        is_fallback=True
                    )
                    fallback_plugins = placeholder._plugins_cache
                    if fallback_plugins:
                        fallbacks[placeholder.pk] += fallback_plugins
                        break

    # These placeholders have no fallback
    non_fallback_phs = [ph for ph in placeholders if ph.pk not in fallbacks]
    # If no plugin is present in non fallback placeholders, create default plugins if enabled)
    if not plugins:
        plugins = create_default_plugins(request, non_fallback_phs, template, lang)
    plugins = downcast_plugins(plugins, non_fallback_phs, request=request)
    # split the plugins up by placeholder
    # Plugins should still be sorted by placeholder
    plugin_groups = dict((key, list(plugins))
                         for key, plugins in groupby(plugins, attrgetter('placeholder_id')))
    all_plugins_groups = plugin_groups.copy()
    for group in plugin_groups:
        plugin_groups[group] = build_plugin_tree(plugin_groups[group])
    groups = fallbacks.copy()
    groups.update(plugin_groups)
    for placeholder in placeholders:
        # This is all the plugins.
        setattr(placeholder, '_all_plugins_cache', all_plugins_groups.get(placeholder.pk, []))
        # This one is only the root plugins.
        setattr(placeholder, '_plugins_cache', groups.get(placeholder.pk, []))



# -*- coding: utf-8 -*
from django.contrib.auth.models import User
# from django.core.urlresolvers import reverse
from django.test import TestCase, Client, override_settings
from cms.api import create_page


class AlternateToolbarTests(TestCase):

    def setUp(self):
        self.client = Client()
        self.user = User.objects.create_superuser(
            username='fred',
            password='test',
            email='test@test.fred',
        )

    def tearDown(self):
        pass

    @override_settings(DEBUG=True)
    def test_toolbar_renders_without_pages(self):
        """
        weird recursion bug, only when DEBUG=True and without any pages
        edge case, but annonying when setting up a new site
        """
        response = self.client.get('/')
        self.assertEqual(response.status_code, 302)
        self.client.login(username='fred', password='test')
        response = self.client.get('/en/admin/')
        self.assertEqual(response.status_code, 200)
        response = self.client.get('/en/?edit')
        self.assertEqual(response.status_code, 200)
        response = self.client.get('/en/not-existing-at-all/')
        self.assertEqual(response.status_code, 404)

    def test_toolbar_renders_has_links(self):
        """
        basic tests. check for some links and if it renders at all
        """
        self.client.login(username='fred', password='test')
        page = create_page('test', 'base.html', 'en')
        page.publish('en')

        url = page.get_absolute_url()
        response = self.client.get(url)
        self.assertRegexpMatches(str(response.content), "\"/en/admin/auth/\"")
        self.assertRegexpMatches(str(response.content), "\"/en/admin/password_change/\"")



from __future__ import unicode_literals

# from django.db.models.signals import post_save, post_delete
from django.conf import settings
from django.dispatch import receiver
# from cms.models import Title
from cms.constants import PUBLISHER_STATE_DIRTY
from cms.signals import post_placeholder_operation, post_publish  # , post_obj_operation

from djangocms_misc.global_untranslated_placeholder.utils import get_untranslated_default_language


if 'djangocms_misc.autopublisher' not in settings.INSTALLED_APPS:

    # published. if not default language, publish default as well
    # if DJANGOCMS_MISC_UNTRANSLATED_MARK_ALL = True, publish all langs (usability thing, mostly)
    @receiver(
        post_publish,
        dispatch_uid="cms_global_untranslated_placeholder_post_publish",
    )
    def post_publish_handler(**kwargs):
        page_instance = kwargs.get('instance', None)
        if not page_instance:
            return
        published_language = kwargs.get('language', settings.LANGUAGE_CODE)
        default_language = get_untranslated_default_language()
        if not published_language == default_language:
            page_instance.publish(default_language)

    # something has changed, if we are not on the default lang, mark current lang/title as as well
    # if DJANGOCMS_MISC_UNTRANSLATED_MARK_ALL = True, mark all languages as dirty!
    @receiver(
        post_placeholder_operation,
        dispatch_uid="cms_global_untranslated_placeholder_post_placeholder_operation",
    )
    def post_ph_operation_handler(sender, operation, request, language, token, origin, **kwargs):
        plugin = None
        if operation == 'change_plugin':
            plugin = kwargs.get('new_plugin', None)
        if not plugin:
            plugin = kwargs.get('plugin', None)
        if not plugin:
            plugin = kwargs.get('plugins', [None, ])[0]
        if plugin:
            placeholder = plugin.placeholder
            if not plugin.language == language:
                # mark current language Title as dirty as well!
                if placeholder.page:
                    title = placeholder.page.get_title_obj(language)
                    title.publisher_state = PUBLISHER_STATE_DIRTY
                    title.save()



from cms.plugin_pool import plugin_pool
from cms.plugin_base import CMSPluginBase
from modeltranslation.admin import TranslationAdmin

from djangocms_misc.basic.admin import LanguageTabsMixin
from djangocms_misc.tests.test_app.models import TestPluginModel


class TestPlugin(LanguageTabsMixin, TranslationAdmin, CMSPluginBase):
    model = TestPluginModel
    render_template = 'test_app/testplugin.html'


plugin_pool.register_plugin(TestPlugin)



from django.shortcuts import redirect


class RedirectFirstSubpageMiddleware(object):

    def __init__(self, get_response):
        self.get_response = get_response
        # One-time configuration and initialization.

    def __call__(self, request):
        # Code to be executed for each request before
        # the view (and later middleware) are called.
        response = self.get_response(request)
        # Code to be executed for each request/response after
        # the view is called.
        return response

    def process_view(self, request, view_func, view_args, view_kwargs):
        if getattr(request, 'current_page', None):
            the_page = request.current_page
            the_redirect = the_page.get_redirect()
            # some more checks if in a cms view!
            if view_func.__name__ == 'details' and "slug" in view_kwargs and the_redirect == "/firstchild":
                if getattr(request.current_page, 'get_child_pages', None):
                    subpages = request.current_page.get_child_pages()
                else:
                    subpages = request.current_page.children.all()
                if len(subpages):
                    return redirect(subpages[0].get_absolute_url(), permanent=True)
        return None



"""URLs to run the tests."""
from django.conf import settings
from django.conf.urls import include, url
from django.conf.urls.i18n import i18n_patterns
from django.contrib import admin
from django.views import static


admin.autodiscover()

urlpatterns = [
]

urlpatterns += i18n_patterns(
    url(r'^admin/', admin.site.urls),
    url(r'^', include('cms.urls')),
)


if settings.DEBUG:
    urlpatterns += [
        url(r'^media/(?P<path>.*)$', static.serve, {'document_root': settings.MEDIA_ROOT}),
    ]



# -*- coding: utf-8 -*-
from django.contrib.auth.models import User
from django.test import TestCase, Client

# compat
import django
if django.VERSION[:2] < (1, 10):
    from django.core.urlresolvers import reverse
else:
    from django.urls import reverse


class AdminStyleTests(TestCase):
    def setUp(self):
        self.client = Client()
        self.user = User.objects.create_superuser(
            username='fred',
            password='test',
            email='test@test.fred',
        )

    def tearDown(self):
        pass

    def test_has_css(self):
        self.client.login(username='fred', password='test')
        url = reverse('admin:index')
        response = self.client.get(url, follow=True)
        self.assertContains(response, 'admin/djangocms_misc/css/admin_style.css')



# -*- coding: utf-8 -*-
# Generated by Django 1.10.7 on 2017-08-05 01:36
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('cms', '0016_auto_20160608_1535'),
        ('test_app', '0001_initial'),
    ]

    operations = [
        migrations.CreateModel(
            name='TestPluginModel',
            fields=[
                ('cmsplugin_ptr', models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, related_name='test_app_testpluginmodel', serialize=False, to='cms.CMSPlugin')),
                ('field1', models.CharField(default='', max_length=64)),
            ],
            options={
                'abstract': False,
            },
            bases=('cms.cmsplugin',),
        ),
        migrations.AlterField(
            model_name='testinlinemodel',
            name='field1',
            field=models.CharField(default='', max_length=64),
        ),
        migrations.AlterField(
            model_name='testmodel',
            name='field1',
            field=models.CharField(default='', max_length=64),
        ),
    ]



from __future__ import unicode_literals

from django.conf import settings


# thx to @wullerot https://gist.github.com/wullerot/9fe3151101e57a9ee6fadb3cababb619
class LanguageTabsMixin(object):
    change_form_template = 'admin/djangocms_misc/modeltranslation_lang_tabs_change_form.html'

    def change_view(self, request, object_id, form_url='', extra_context=None):
        context = extra_context or {}
        context['tab_languages'] = settings.LANGUAGES
        return super(LanguageTabsMixin, self).change_view(
            request,
            object_id=object_id,
            form_url=form_url,
            extra_context=context
        )

    def add_view(self, request, form_url='', extra_context=None):
        context = extra_context or {}
        context['tab_languages'] = settings.LANGUAGES
        return super(LanguageTabsMixin, self).add_view(
            request,
            form_url=form_url,
            extra_context=context
        )
